{$WARNINGS OFF}
unit CommonMemoryUnit;

interface

uses
  Windows, Messages, SysUtils, Classes, Dialogs;

type
TCommMemNT = class(TObject)
private
  FMemPtr:pointer;
  FhProcess:THandle;
  FdwProcessId:DWORD;
  FSize:integer;
  FPosition : Cardinal;
protected
  procedure Open(AllocSize:integer);
  procedure Close;
public
  constructor Create(hTarget:HWND;AllocSize:integer);
  destructor Destroy;override;
  procedure ZeroClear;
  function Write(offset:integer;Source:pointer;Length:DWORD): Cardinal; overload;
  function Write(Source:pointer;Length:NativeUInt): Cardinal; overload;
  function Read(offset:integer;Destination:pointer;Length:DWORD): Cardinal; overload;
  function Read(Destination:pointer;Length:NativeUInt): Cardinal; overload;
  procedure ReOpen(ReAllocSize:integer);
  property MemPtr:pointer read FMemPtr;
  property Size:Integer read FSize;
  property Position : Cardinal read FPosition write FPosition;
end;


implementation


{ TCommMemNT }

constructor TCommMemNT.Create(hTarget: HWND; AllocSize: integer);
begin
  GetWindowThreadProcessId(hTarget,@FdwProcessId);
  FhProcess := OpenProcess(PROCESS_VM_OPERATION or PROCESS_VM_READ or
                          PROCESS_VM_WRITE,false,FdwProcessId);
  Open(AllocSize);
  FSize := AllocSize;
  FPosition := 0;
end;

destructor TCommMemNT.Destroy;
begin
  Close;
  CloseHandle(FhProcess);
  inherited;
end;

procedure TCommMemNT.Open(AllocSize: integer);
begin
  FMemPtr := VirtualAllocEx(FhProcess,nil,AllocSize,
                           MEM_RESERVE or MEM_COMMIT,PAGE_READWRITE);
  FPosition := 0;
end;

procedure TCommMemNT.Close;
begin
  VirtualFreeEx(FhProcess,FMemPtr,0,MEM_RELEASE);
end;

procedure TCommMemNT.ReOpen(ReAllocSize: integer);
var
  ptr:pointer;
begin
  GetMem(ptr,FSize);
  Read(0,ptr,FSize);
  Close;
  Open(ReAllocSize);
  if FSize <= ReAllocSize then
    Write(0,ptr,FSize)
  else
    Write(0,ptr,ReAllocSize);
  FreeMem(ptr);
  FSize := ReAllocSize;
  FPosition := 0;
end;

function TCommMemNT.Read(offset: integer; Destination: pointer;
    Length: DWORD): Cardinal;
var
  numRead:NativeUInt;
begin
  Result := 0;
  if (offset+Length)>FSize then begin
    ShowMessage('ǂݍݔ͈͂TCY𒴂Ă܂');
    exit;
  end;

  ReadProcessMemory(FhProcess,pointer(integer(FMemPtr)+offset),Destination,
      Length,numRead);
  FPosition := FPosition + numRead + 1;
  Result := numRead;
end;

function TCommMemNT.Read(Destination: pointer; Length: NativeUInt): Cardinal;
var
  numRead:NativeUInt;
begin
  Result := 0;
  if (FPosition+Length)>FSize then begin
    ShowMessage('ǂݍݔ͈͂TCY𒴂Ă܂');
    exit;
  end;

  ReadProcessMemory(FhProcess,pointer(integer(FMemPtr)+FPosition),Destination,
      Length,numRead);
  FPosition := FPosition + numRead + 1;
  Result := numRead;
end;

function TCommMemNT.Write(offset: integer; Source: pointer;
    Length: DWORD): Cardinal;
var
  numWrite:NativeUInt;
begin
  Result := 0;
  if (offset+Length)>FSize then begin
    ShowMessage('ݔ͈͂TCY𒴂Ă܂');
    exit;
  end;

  WriteProcessMemory(FhProcess,pointer(integer(FMemPtr)+offset),Source,
      Length,numWrite);
  FPosition := FPosition + numWrite + 1;
  Result := numWrite;
end;

function TCommMemNT.Write(Source: pointer; Length: NativeUInt): Cardinal;
var
  numWrite:NativeUInt;
begin
  Result := 0;
  if (FPosition+Length)>FSize then begin
    ShowMessage('ݔ͈͂TCY𒴂Ă܂');
    exit;
  end;

  WriteProcessMemory(FhProcess,pointer(integer(FMemPtr)+FPosition),Source,
      Length,numWrite);
  FPosition := FPosition + numWrite + 1;
  Result := numWrite;
end;

procedure TCommMemNT.ZeroClear;
var
  ptr:pointer;
begin
  GetMem(ptr,FSize);
  ZeroMemory(ptr,FSize);
  Write(0,ptr,FSize);
  FreeMem(ptr);
  FPosition := 0;
end;

end.


